@@ -23,16 +23,22 @@
 #include "sybdb.h"
 
 #ifdef __cplusplus
-extern "C" {
+extern "C"
+{
+#if 0
+}
+#endif
 #endif
 
-static char  rcsid_sybfront_h [ ] =
-         "$Id: sybfront.h,v 1.1.1.1 2001/10/12 23:28:55 brianb Exp $";
-static void *no_unused_sybfront_h_warn[]={rcsid_sybfront_h, no_unused_sybfront_h_warn};
+static const char rcsid_sybfront_h[] = "$Id: sybfront.h,v 1.3 2004/10/28 12:42:12 freddy77 Exp $";
+static const void *const no_unused_sybfront_h_warn[] = { rcsid_sybfront_h, no_unused_sybfront_h_warn };
 
 
 
 #ifdef __cplusplus
+#if 0
+{
+#endif
 }
 #endif
 
