@@ -29,12 +29,12 @@ extern "C"
 }
 #endif
 
-static const char rcsid_tdssrv_h[] = "$Id: tdssrv.h,v 1.7 2004/10/28 12:42:12 freddy77 Exp $";
+static const char rcsid_tdssrv_h[] = "$Id: tdssrv.h,v 1.10 2008/01/07 14:07:21 freddy77 Exp $";
 static const void *const no_unused_tdssrv_h_warn[] = { rcsid_tdssrv_h, no_unused_tdssrv_h_warn };
 
 /* login.c */
 unsigned char *tds7_decrypt_pass(const unsigned char *crypt_pass, int len, unsigned char *clear_pass);
-TDSSOCKET *tds_listen(int ip_port);
+TDSSOCKET *tds_listen(TDSCONTEXT * ctx, int ip_port);
 void tds_read_login(TDSSOCKET * tds, TDSLOGIN * login);
 int tds7_read_login(TDSSOCKET * tds, TDSLOGIN * login);
 TDSLOGIN *tds_alloc_read_login(TDSSOCKET * tds);
@@ -52,15 +52,16 @@ void tds_send_eed(TDSSOCKET * tds, int msgno, int msgstate, int severity, char *
 void tds_send_err(TDSSOCKET * tds, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr);
 void tds_send_capabilities_token(TDSSOCKET * tds);
 /* TODO remove, use tds_send_done */
-void tds_send_253_token(TDSSOCKET * tds, TDS_SMALLINT flags, TDS_INT numrows);
+void tds_send_done_token(TDSSOCKET * tds, TDS_SMALLINT flags, TDS_INT numrows);
 void tds_send_done(TDSSOCKET * tds, int token, TDS_SMALLINT flags, TDS_INT numrows);
-void tds_send_174_token(TDSSOCKET * tds, TDS_SMALLINT numcols);
+void tds_send_control_token(TDSSOCKET * tds, TDS_SMALLINT numcols);
 void tds_send_col_name(TDSSOCKET * tds, TDSRESULTINFO * resinfo);
 void tds_send_col_info(TDSSOCKET * tds, TDSRESULTINFO * resinfo);
 void tds_send_result(TDSSOCKET * tds, TDSRESULTINFO * resinfo);
 void tds7_send_result(TDSSOCKET * tds, TDSRESULTINFO * resinfo);
 void tds_send_table_header(TDSSOCKET * tds, TDSRESULTINFO * resinfo);
 void tds_send_row(TDSSOCKET * tds, TDSRESULTINFO * resinfo);
+void tds8_send_prelogin(TDSSOCKET * tds);
 
 #if 0
 {
