@@ -2,7 +2,7 @@
 #ifndef COMMON_h
 #define COMMON_h
 
-static char rcsid_common_h[] = "$Id: common.h,v 1.10 2004/07/21 19:28:01 freddy77 Exp $";
+static char rcsid_common_h[] = "$Id: common.h,v 1.11 2008/07/06 16:44:24 jklowden Exp $";
 static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_warn };
 
 extern char SERVER[512];
@@ -17,6 +17,8 @@ typedef struct
 	char DATABASE[512];
 	char USER[512];
 	char PASSWORD[512];
+	char fverbose;
+	int maxlength; 
 } COMMON_PWD;
 extern COMMON_PWD common_pwd;
 
@@ -27,7 +29,11 @@ extern int clientmsg_cb_invoked;
 extern int servermsg_cb_invoked;
 
 CS_RETCODE try_ctlogin(CS_CONTEXT ** ctx, CS_CONNECTION ** conn, CS_COMMAND ** cmd, int verbose);
+CS_RETCODE try_ctlogin_with_options(int argc, char **argv, CS_CONTEXT ** ctx, CS_CONNECTION ** conn, CS_COMMAND ** cmd, 
+				    int verbose);
+
 CS_RETCODE try_ctlogout(CS_CONTEXT * ctx, CS_CONNECTION * conn, CS_COMMAND * cmd, int verbose);
+
 CS_RETCODE run_command(CS_COMMAND * cmd, const char *sql);
 CS_RETCODE cslibmsg_cb(CS_CONTEXT * connection, CS_CLIENTMSG * errmsg);
 CS_RETCODE clientmsg_cb(CS_CONTEXT * context, CS_CONNECTION * connection, CS_CLIENTMSG * errmsg);
