@@ -3,7 +3,7 @@
 /* Test for store procedure and params */
 /* Test from Tom Rogers */
 
-static char software_version[] = "$Id: params.c,v 1.7 2005/03/29 15:19:36 freddy77 Exp $";
+static char software_version[] = "$Id: params.c,v 1.12 2010/07/05 09:20:33 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 /* SP definition */
@@ -30,62 +30,32 @@ Test(int bind_before)
 	SQLLEN cbReturnCode = 0, cbInParam = 0, cbOutParam = 0;
 	SQLLEN cbOutString = SQL_NTS;
 
-	Connect();
+	odbc_connect();
 
 	/* drop proc */
-	if (CommandWithResult(Statement, "drop proc spTestProc") != SQL_SUCCESS)
-		printf("Unable to execute statement\n");
+	odbc_command("IF OBJECT_ID('spTestProc') IS NOT NULL DROP PROC spTestProc");
 
 	/* create proc */
-	Command(Statement, sp_define);
+	odbc_command(sp_define);
 
-	if (!bind_before) {
-		if (SQLPrepare(Statement, (SQLCHAR *) SP_TEXT, strlen(SP_TEXT)) != SQL_SUCCESS) {
-			fprintf(stderr, "Unable to prepare statement\n");
-			return 1;
-		}
-	}
+	if (!bind_before)
+		CHKPrepare((SQLCHAR *) SP_TEXT, strlen(SP_TEXT), "S");
 
-	if (SQLBindParameter(Statement, 1, SQL_PARAM_OUTPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &ReturnCode, 0, &cbReturnCode) !=
-	    SQL_SUCCESS) {
-		fprintf(stderr, "Unable to bind input parameter\n");
-		return 1;
-	}
-
-	if (SQLBindParameter(Statement, 2, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &InParam, 0, &cbInParam) !=
-	    SQL_SUCCESS) {
-		fprintf(stderr, "Unable to bind input parameter\n");
-		return 1;
-	}
-
-	if (SQLBindParameter(Statement, 3, SQL_PARAM_OUTPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &OutParam, 0, &cbOutParam) !=
-	    SQL_SUCCESS) {
-		fprintf(stderr, "Unable to bind input parameter\n");
-		return 1;
-	}
+	CHKBindParameter(1, SQL_PARAM_OUTPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &ReturnCode, 0, &cbReturnCode, "S");
+	CHKBindParameter(2, SQL_PARAM_INPUT,  SQL_C_SSHORT, SQL_INTEGER, 0, 0, &InParam,    0, &cbInParam,    "S");
+	CHKBindParameter(3, SQL_PARAM_OUTPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &OutParam,   0, &cbOutParam,   "S");
 
 	OutString[0] = '\0';
 	strcpy(OutString, "Test");	/* Comment this line and we get an error!  Why? */
-	if (SQLBindParameter
-	    (Statement, 4, SQL_PARAM_OUTPUT, SQL_C_CHAR, SQL_VARCHAR, OUTSTRING_LEN, 0, OutString, OUTSTRING_LEN,
-	     &cbOutString) != SQL_SUCCESS) {
-		fprintf(stderr, "Unable to bind input parameter\n");
-		return 1;
-	}
+	CHKBindParameter(4, SQL_PARAM_OUTPUT, SQL_C_CHAR, SQL_VARCHAR, OUTSTRING_LEN, 0, OutString, 
+	    OUTSTRING_LEN, &cbOutString, "S");
 
-	if (bind_before) {
-		if (SQLPrepare(Statement, (SQLCHAR *) SP_TEXT, strlen(SP_TEXT)) != SQL_SUCCESS) {
-			fprintf(stderr, "Unable to prepare statement\n");
-			return 1;
-		}
-	}
+	if (bind_before)
+		CHKPrepare((SQLCHAR *) SP_TEXT, strlen(SP_TEXT), "S");
 
-	if (SQLExecute(Statement) != SQL_SUCCESS) {
-		fprintf(stderr, "Unable to execute statement\n");
-		return 1;
-	}
+	CHKExecute("S");
 
-	Command(Statement, "drop proc spTestProc");
+	odbc_command("DROP PROC spTestProc");
 
 	printf("Output:\n");
 	printf("   Return Code = %d\n", (int) ReturnCode);
@@ -103,7 +73,7 @@ Test(int bind_before)
 		return 1;
 	}
 
-	Disconnect();
+	odbc_disconnect();
 	return 0;
 }
 
