@@ -10,7 +10,7 @@
 
 #include "common.h"
 
-static char software_version[] = "$Id: testodbc.c,v 1.9 2005/06/29 07:21:24 freddy77 Exp $";
+static char software_version[] = "$Id: testodbc.c,v 1.15 2010/07/05 09:20:33 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 #ifdef DEBUG
@@ -36,76 +36,13 @@ typedef struct
 	const char *description;
 } DbTestEntry;
 
-/*
- * Output ODBC errors.
- */
-static void
-DispODBCErrs(SQLHENV envHandle, SQLHDBC connHandle, SQLHSTMT statementHandle)
-{
-	SQLCHAR buffer[256];
-	SQLCHAR sqlState[16];
-
-	/* Statement errors */
-	if (statementHandle) {
-		while (SQLError(envHandle, connHandle, statementHandle, sqlState, 0, buffer, sizeof(buffer), 0) == SQL_SUCCESS) {
-			AB_ERROR(("%s, SQLSTATE=%s", buffer, sqlState));
-		}
-	}
-
-	/* Connection errors */
-	while (SQLError(envHandle, connHandle, SQL_NULL_HSTMT, sqlState, 0, buffer, sizeof(buffer), 0) == SQL_SUCCESS) {
-		AB_ERROR(("%s, SQLSTATE=%s", buffer, sqlState));
-	}
-
-	/* Environmental errors */
-	while (SQLError(envHandle, SQL_NULL_HDBC, SQL_NULL_HSTMT, sqlState, 0, buffer, sizeof(buffer), 0) == SQL_SUCCESS) {
-		AB_ERROR(("%s, SQLSTATE=%s", buffer, sqlState));
-	}
-}
-
-/*
- * Output ODBC diagnostics. Only used for 'raw' ODBC tests.
- */
-static void
-DispODBCDiags(SQLHSTMT statementHandle)
-{
-	SQLSMALLINT recNumber;
-	SQLCHAR sqlState[10];
-	SQLINTEGER nativeError = -99;
-	SQLCHAR messageText[500];
-	SQLSMALLINT bufferLength = 500;
-	SQLSMALLINT textLength = -99;
-	SQLRETURN status;
-
-	recNumber = 1;
-
-	AB_FUNCT(("DispODBCDiags (in)"));
-
-	do {
-		status = SQLGetDiagRec(SQL_HANDLE_STMT, statementHandle, recNumber,
-				       sqlState, &nativeError, messageText, bufferLength, &textLength);
-		if (status != SQL_SUCCESS) {
-			/* No data mean normal end of iteration. Anything else is error. */
-			if (status != SQL_NO_DATA) {
-				AB_ERROR(("SQLGetDiagRec status is %d", status));
-			}
-			break;
-		}
-		printf("DIAG #%d, sqlState=%s, nativeError=%d, message=%s\n", recNumber, sqlState, (int) nativeError, messageText);
-		recNumber++;
-	} while (status == SQL_SUCCESS);
-
-	AB_FUNCT(("DispODBCDiags (out)"));
-}
-
 /*
  * Test that makes a parameterized ODBC query using SQLPrepare and SQLExecute
  */
 static int
 TestRawODBCPreparedQuery(void)
 {
-	SQLRETURN status;
-	SQLCHAR queryString[200];
+	SQLCHAR *queryString;
 	SQLLEN lenOrInd = 0;
 	SQLSMALLINT supplierId = 4;
 	int count;
@@ -114,11 +51,11 @@ TestRawODBCPreparedQuery(void)
 
 	/* INIT */
 
-	Connect();
+	odbc_connect();
 
 	/* MAKE QUERY */
 
-	Command(Statement, "CREATE TABLE #Products ("
+	odbc_command("CREATE TABLE #Products ("
 		"ProductID int NOT NULL ,"
 		"ProductName varchar (40) ,"
 		"SupplierID int NULL ,"
@@ -135,38 +72,19 @@ TestRawODBCPreparedQuery(void)
 		"INSERT INTO #Products(ProductID,ProductName,SupplierID,CategoryID,QuantityPerUnit,UnitPrice,UnitsInStock,UnitsOnOrder,ReorderLevel,Discontinued) VALUES(74,'Longlife Tofu',4,7,'5 kg pkg.',10.00,4,20,5,0) "
 		"INSERT INTO #Products(ProductID,ProductName,SupplierID,CategoryID,QuantityPerUnit,UnitPrice,UnitsInStock,UnitsOnOrder,ReorderLevel,Discontinued) VALUES(11,'Queso Cabrales',5,4,'1 kg pkg.',21.00,22,30,30,0) "
 		"INSERT INTO #Products(ProductID,ProductName,SupplierID,CategoryID,QuantityPerUnit,UnitPrice,UnitsInStock,UnitsOnOrder,ReorderLevel,Discontinued) VALUES(12,'Queso Manchego La Pastora',5,4,'10 - 500 g pkgs.',38.00,86,0,0,0)");
-	while (SQLMoreResults(Statement) == SQL_SUCCESS);
+	while (SQLMoreResults(odbc_stmt) == SQL_SUCCESS);
 
-	strcpy((char *) (queryString), "SELECT * FROM #Products WHERE SupplierID = ?");
+	queryString = (SQLCHAR *) "SELECT * FROM #Products WHERE SupplierID = ?";
 
-	status = SQLBindParameter(Statement, 1, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &supplierId, 0, &lenOrInd);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("SQLBindParameter failed"));
-		DispODBCErrs(Environment, Connection, Statement);
-		DispODBCDiags(Statement);
-		AB_FUNCT(("TestRawODBCPreparedQuery (out): error"));
-		return FALSE;
-	}
+	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &supplierId, 0, &lenOrInd, "S");
 
-	status = SQLPrepare(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("Prepare failed"));
-		AB_FUNCT(("TestRawODBCPreparedQuery (out): error"));
-		return FALSE;
-	}
+	CHKPrepare(queryString, SQL_NTS, "S");
 
-	status = SQLExecute(Statement);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("Execute failed"));
-		DispODBCErrs(Environment, Connection, Statement);
-		DispODBCDiags(Statement);
-		AB_FUNCT(("TestRawODBCPreparedQuery (out): error"));
-		return FALSE;
-	}
+	CHKExecute("S");
 
 	count = 0;
 
-	while (SQLFetch(Statement) == SQL_SUCCESS) {
+	while (SQLFetch(odbc_stmt) == SQL_SUCCESS) {
 		count++;
 	}
 	AB_PRINT(("Got %d rows", count));
@@ -183,7 +101,7 @@ TestRawODBCPreparedQuery(void)
 
 	/* CLOSEDOWN */
 
-	Disconnect();
+	odbc_disconnect();
 
 	AB_FUNCT(("TestRawODBCPreparedQuery (out): ok"));
 	return TRUE;
@@ -195,8 +113,6 @@ TestRawODBCPreparedQuery(void)
 static int
 TestRawODBCDirectQuery(void)
 {
-	SQLRETURN status;
-	SQLCHAR queryString[200];
 	SQLLEN lenOrInd = 0;
 	SQLSMALLINT supplierId = 1;
 	int count;
@@ -205,11 +121,11 @@ TestRawODBCDirectQuery(void)
 
 	/* INIT */
 
-	Connect();
+	odbc_connect();
 
 	/* MAKE QUERY */
 
-	Command(Statement, "CREATE TABLE #Products ("
+	odbc_command("CREATE TABLE #Products ("
 		"ProductID int NOT NULL ,"
 		"ProductName varchar (40) ,"
 		"SupplierID int NULL ,"
@@ -226,31 +142,15 @@ TestRawODBCDirectQuery(void)
 		"INSERT INTO #Products(ProductID,ProductName,SupplierID,CategoryID,QuantityPerUnit,UnitPrice,UnitsInStock,UnitsOnOrder,ReorderLevel,Discontinued) VALUES(3,'Aniseed Syrup',1,2,'12 - 550 ml bottles',10.00,13,70,25,0) "
 		"INSERT INTO #Products(ProductID,ProductName,SupplierID,CategoryID,QuantityPerUnit,UnitPrice,UnitsInStock,UnitsOnOrder,ReorderLevel,Discontinued) VALUES(4,'Chef Anton''s Cajun Seasoning',2,2,'48 - 6 oz jars',22.00,53,0,0,0) "
 		"INSERT INTO #Products(ProductID,ProductName,SupplierID,CategoryID,QuantityPerUnit,UnitPrice,UnitsInStock,UnitsOnOrder,ReorderLevel,Discontinued) VALUES(5,'Chef Anton''s Gumbo Mix',2,2,'36 boxes',21.35,0,0,0,1) ");
-	while (SQLMoreResults(Statement) == SQL_SUCCESS);
+	while (SQLMoreResults(odbc_stmt) == SQL_SUCCESS);
 
-	strcpy((char *) (queryString), "SELECT * FROM #Products WHERE SupplierID = ?");
+	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &supplierId, 0, &lenOrInd, "S");
 
-	status = SQLBindParameter(Statement, 1, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &supplierId, 0, &lenOrInd);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("SQLBindParameter failed"));
-		DispODBCErrs(Environment, Connection, Statement);
-		DispODBCDiags(Statement);
-		AB_FUNCT(("TestRawODBCDirectQuery (out): error"));
-		return FALSE;
-	}
-
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("Execute failed"));
-		DispODBCErrs(Environment, Connection, Statement);
-		DispODBCDiags(Statement);
-		AB_FUNCT(("TestRawODBCDirectQuery (out): error"));
-		return FALSE;
-	}
+	CHKExecDirect((SQLCHAR *) "SELECT * FROM #Products WHERE SupplierID = ?", SQL_NTS, "S");
 
 	count = 0;
 
-	while (SQLFetch(Statement) == SQL_SUCCESS) {
+	while (SQLFetch(odbc_stmt) == SQL_SUCCESS) {
 		count++;
 	}
 	AB_PRINT(("Got %d rows", count));
@@ -267,7 +167,7 @@ TestRawODBCDirectQuery(void)
 
 	/* CLOSEDOWN */
 
-	Disconnect();
+	odbc_disconnect();
 
 	AB_FUNCT(("TestRawODBCDirectQuery (out): ok"));
 	return TRUE;
@@ -282,7 +182,7 @@ TestRawODBCGuid(void)
 {
 	SQLRETURN status;
 
-	SQLCHAR queryString[300];
+	SQLCHAR *queryString;
 	SQLLEN lenOrInd;
 	SQLSMALLINT age;
 	SQLCHAR guid[40];
@@ -293,120 +193,81 @@ TestRawODBCGuid(void)
 
 	AB_FUNCT(("TestRawODBCGuid (in)"));
 
-	Connect();
+	odbc_connect();
 	
-	if (!db_is_microsoft()) {
-		Disconnect();
+	if (!odbc_db_is_microsoft()) {
+		odbc_disconnect();
 		return TRUE;
 	}
 
 	AB_PRINT(("Creating #pet table"));
 
-	strcpy((char *) (queryString), "CREATE TABLE #pet (name VARCHAR(20), owner VARCHAR(20), "
-	       "species VARCHAR(20), sex CHAR(1), age INTEGER, " "guid UNIQUEIDENTIFIER DEFAULT NEWID() ); ");
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS && status != SQL_NO_DATA) {
-		AB_ERROR(("Create table failed"));
-		goto odbcfail;
-	}
+	queryString = (SQLCHAR *) "CREATE TABLE #pet (name VARCHAR(20), owner VARCHAR(20), "
+	       "species VARCHAR(20), sex CHAR(1), age INTEGER, " "guid UNIQUEIDENTIFIER DEFAULT NEWID() ); ";
+	CHKExecDirect(queryString, SQL_NTS, "SNo");
 
-	CommandWithResult(Statement, "DROP PROC GetGUIDRows");
+	odbc_command_with_result(odbc_stmt, "DROP PROC GetGUIDRows");
 
 	AB_PRINT(("Creating stored proc GetGUIDRows"));
 
-	strcpy((char *) (queryString), "CREATE PROCEDURE GetGUIDRows (@guidpar uniqueidentifier) AS \
-                SELECT name, guid FROM #pet WHERE guid = @guidpar");
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS && status != SQL_NO_DATA) {
-		AB_ERROR(("Create procedure failed"));
-		goto odbcfail;
-	}
+	queryString = (SQLCHAR *) "CREATE PROCEDURE GetGUIDRows (@guidpar uniqueidentifier) AS \
+                SELECT name, guid FROM #pet WHERE guid = @guidpar";
+	CHKExecDirect(queryString, SQL_NTS, "SNo");
 
 	AB_PRINT(("Insert row 1"));
 
-	strcpy((char *) (queryString), "INSERT INTO #pet( name, owner, species, sex, age ) \
-                         VALUES ( 'Fang', 'Mike', 'dog', 'm', 12 );");
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("Insert row 1 failed"));
-		goto odbcfail;
-	}
+	queryString = (SQLCHAR *) "INSERT INTO #pet( name, owner, species, sex, age ) \
+                         VALUES ( 'Fang', 'Mike', 'dog', 'm', 12 );";
+	CHKExecDirect(queryString, SQL_NTS, "S");
 
 	AB_PRINT(("Insert row 2"));
 
 	/*
 	 * Ok - new row with explicit GUID, but parameterised age.
 	 */
-	strcpy((char *) (queryString), "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
+	queryString = (SQLCHAR *) "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
                          VALUES ( 'Splash', 'Dan', 'fish', 'm', ?, \
-                         '12345678-1234-1234-1234-123456789012' );");
+                         '12345678-1234-1234-1234-123456789012' );";
 
 	lenOrInd = 0;
 	age = 3;
-	if (SQLBindParameter(Statement, 1, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &age, 0, &lenOrInd)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLBindParameter failed"));
-		goto odbcfail;
-	}
+	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, 0, 0, &age, 0, &lenOrInd, "S");
 
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("Insert row 2 failed"));
-		goto odbcfail;
-	}
-	if (SQLFreeStmt(Statement, SQL_CLOSE) != SQL_SUCCESS) {
-		AB_ERROR(("Free statement failed (5)"));
-		goto odbcfail;
-	}
+	CHKExecDirect(queryString, SQL_NTS, "S");
+	CHKFreeStmt(SQL_CLOSE, "S");
 
 	AB_PRINT(("Insert row 3"));
 	/*
 	 * Ok - new row with parameterised GUID.
 	 */
-	strcpy((char *) (queryString), "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
-                         VALUES ( 'Woof', 'Tom', 'cat', 'f', 2, ? );");
+	queryString = (SQLCHAR *) "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
+                         VALUES ( 'Woof', 'Tom', 'cat', 'f', 2, ? );";
 
 	lenOrInd = SQL_NTS;
 	strcpy((char *) (guid), "87654321-4321-4321-4321-123456789abc");
 
-	if (SQLBindParameter(Statement, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_GUID, 0, 0, guid, 0, &lenOrInd)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLBindParameter failed"));
-		goto odbcfail;
-	}
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("Insert row 3 failed"));
-		goto odbcfail;
-	}
+	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_GUID, 0, 0, guid, 0, &lenOrInd, "S");
+	CHKExecDirect(queryString, SQL_NTS, "S");
 
 	AB_PRINT(("Insert row 4"));
 	/*
 	 * Ok - new row with parameterised GUID.
 	 */
-	strcpy((char *) (queryString), "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
-                         VALUES ( 'Spike', 'Diane', 'pig', 'f', 4, ? );");
+	queryString = (SQLCHAR *) "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
+                         VALUES ( 'Spike', 'Diane', 'pig', 'f', 4, ? );";
 
 	lenOrInd = SQL_NTS;
 	strcpy((char *) (guid), "1234abcd-abcd-abcd-abcd-123456789abc");
 
-	if (SQLBindParameter(Statement, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 36, 0, guid, 0, &lenOrInd)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLBindParameter failed"));
-		goto odbcfail;
-	}
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("Insert row 4 failed"));
-		goto odbcfail;
-	}
+	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 36, 0, guid, 0, &lenOrInd, "S");
+	CHKExecDirect(queryString, SQL_NTS, "S");
 
 	AB_PRINT(("Insert row 5"));
 	/*
 	 * Ok - new row with parameterised GUID.
 	 */
-	strcpy((char *) (queryString), "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
-                         VALUES ( 'Fluffy', 'Sam', 'dragon', 'm', 16, ? );");
+	queryString = (SQLCHAR *) "INSERT INTO #pet( name, owner, species, sex, age, guid ) \
+                         VALUES ( 'Fluffy', 'Sam', 'dragon', 'm', 16, ? );";
 
 	sqlguid.Data1 = 0xaabbccdd;
 	sqlguid.Data2 = 0xeeff;
@@ -423,12 +284,8 @@ TestRawODBCGuid(void)
 	lenOrInd = 16;
 	strcpy((char *) (guid), "1234abcd-abcd-abcd-abcd-123456789abc");
 
-	if (SQLBindParameter(Statement, 1, SQL_PARAM_INPUT, SQL_C_GUID, SQL_GUID, 16, 0, &sqlguid, 16, &lenOrInd)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLBindParameter failed"));
-		goto odbcfail;
-	}
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
+	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_GUID, SQL_GUID, 16, 0, &sqlguid, 16, &lenOrInd, "S");
+	status = SQLExecDirect(odbc_stmt, queryString, SQL_NTS);
 	if (status != SQL_SUCCESS) {
 		AB_ERROR(("Insert row 5 failed"));
 		AB_ERROR(("Sadly this was expected in *nix ODBC. Carry on."));
@@ -438,24 +295,12 @@ TestRawODBCGuid(void)
 	 * Now retrieve rows - especially GUID column values.
 	 */
 	AB_PRINT(("retrieving name and guid"));
-	strcpy((char *) (queryString), "SELECT name, guid FROM #pet");
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("SELECT failed"));
-		goto odbcfail;
-	}
-	while (SQLFetch(Statement) == SQL_SUCCESS) {
+	queryString = (SQLCHAR *) "SELECT name, guid FROM #pet";
+	CHKExecDirect(queryString, SQL_NTS, "S");
+	while (SQLFetch(odbc_stmt) == SQL_SUCCESS) {
 		count++;
-		if (SQLGetData(Statement, 1, SQL_CHAR, name, 20, 0)
-		    != SQL_SUCCESS) {
-			AB_ERROR(("Get row %d, name column failed", count));
-			goto odbcfail;
-		}
-		if (SQLGetData(Statement, 2, SQL_CHAR, guid, 37, 0)
-		    != SQL_SUCCESS) {
-			AB_ERROR(("Get row %d, guid column failed", count));
-			goto odbcfail;
-		}
+		CHKGetData(1, SQL_CHAR, name, 20, 0, "S");
+		CHKGetData(2, SQL_CHAR, guid, 37, 0, "S");
 
 		AB_PRINT(("name: %-10s guid: %s", name, guid));
 	}
@@ -464,15 +309,7 @@ TestRawODBCGuid(void)
 	 * Realloc cursor handle - (Windows ODBC considers it an invalid cursor
 	 * state if we try SELECT again).
 	 */
-	if (SQLFreeStmt(Statement, SQL_CLOSE) != SQL_SUCCESS) {
-		AB_ERROR(("Free statement failed (5)"));
-		goto odbcfail;
-	}
-	if (SQLAllocHandle(SQL_HANDLE_STMT, Connection, &Statement)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLAllocStmt failed(1)"));
-		goto odbcfail;
-	}
+	odbc_reset_statement();
 
 
 	/*
@@ -480,24 +317,12 @@ TestRawODBCGuid(void)
 	 */
 
 	AB_PRINT(("retrieving name and guid again"));
-	strcpy((char *) (queryString), "SELECT name, guid FROM #pet");
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("SELECT failed"));
-		goto odbcfail;
-	}
-	while (SQLFetch(Statement) == SQL_SUCCESS) {
+	queryString = (SQLCHAR *) "SELECT name, guid FROM #pet";
+	CHKExecDirect(queryString, SQL_NTS, "S");
+	while (CHKFetch("SNo") == SQL_SUCCESS) {
 		count++;
-		if (SQLGetData(Statement, 1, SQL_CHAR, name, 20, 0)
-		    != SQL_SUCCESS) {
-			AB_ERROR(("Get row %d, name column failed", count));
-			goto odbcfail;
-		}
-		if (SQLGetData(Statement, 2, SQL_GUID, &sqlguid, 16, 0)
-		    != SQL_SUCCESS) {
-			AB_ERROR(("Get row %d, guid column failed", count));
-			goto odbcfail;
-		}
+		CHKGetData(1, SQL_CHAR, name, 20, 0, "S");
+		CHKGetData(2, SQL_GUID, &sqlguid, 16, 0, "S");
 
 		AB_PRINT(("%-10s %08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X",
 			  name,
@@ -511,47 +336,23 @@ TestRawODBCGuid(void)
 	 * Realloc cursor handle - (Windows ODBC considers it an invalid cursor
 	 * state if we try SELECT again).
 	 */
-	if (SQLFreeStmt(Statement, SQL_CLOSE) != SQL_SUCCESS) {
-		AB_ERROR(("Free statement failed (5)"));
-		goto odbcfail;
-	}
-	if (SQLAllocHandle(SQL_HANDLE_STMT, Connection, &Statement)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLAllocStmt failed(1)"));
-		goto odbcfail;
-	}
+	odbc_reset_statement();
 
 	/*
 	 * Now retrieve rows via stored procedure passing GUID as param.
 	 */
 	AB_PRINT(("retrieving name and guid"));
 
-	strcpy((char *) (queryString), "{call GetGUIDRows(?)}");
+	queryString = (SQLCHAR *) "{call GetGUIDRows(?)}";
 	lenOrInd = SQL_NTS;
 	strcpy((char *) (guid), "87654321-4321-4321-4321-123456789abc");
 
-	if (SQLBindParameter(Statement, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_GUID, 0, 0, guid, 0, &lenOrInd)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLBindParameter failed"));
-		goto odbcfail;
-	}
-	status = SQLExecDirect(Statement, queryString, SQL_NTS);
-	if (status != SQL_SUCCESS) {
-		AB_ERROR(("SELECT failed"));
-		goto odbcfail;
-	}
-	while (SQLFetch(Statement) == SQL_SUCCESS) {
+	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_GUID, 0, 0, guid, 0, &lenOrInd, "S");
+	CHKExecDirect(queryString, SQL_NTS, "S");
+	while (SQLFetch(odbc_stmt) == SQL_SUCCESS) {
 		count++;
-		if (SQLGetData(Statement, 1, SQL_CHAR, name, 20, 0)
-		    != SQL_SUCCESS) {
-			AB_ERROR(("Get row %d, name column failed", count));
-			goto odbcfail;
-		}
-		if (SQLGetData(Statement, 2, SQL_CHAR, guid, 37, 0)
-		    != SQL_SUCCESS) {
-			AB_ERROR(("Get row %d, guid column failed", count));
-			goto odbcfail;
-		}
+		CHKGetData(1, SQL_CHAR, name, 20, 0, "S");
+		CHKGetData(2, SQL_CHAR, guid, 37, 0, "S");
 
 		AB_PRINT(("%-10s %s", name, guid));
 	}
@@ -560,31 +361,17 @@ TestRawODBCGuid(void)
 	 * Realloc cursor handle - (Windows ODBC considers it an invalid cursor
 	 * state after a previous SELECT has occurred).
 	 */
-	if (SQLFreeStmt(Statement, SQL_CLOSE) != SQL_SUCCESS) {
-		AB_ERROR(("Free statement failed (5)"));
-		goto odbcfail;
-	}
-	if (SQLAllocHandle(SQL_HANDLE_STMT, Connection, &Statement)
-	    != SQL_SUCCESS) {
-		AB_ERROR(("SQLAllocStmt failed(1)"));
-		goto odbcfail;
-	}
+	odbc_reset_statement();
 
 	/* cleanup */
-	CommandWithResult(Statement, "DROP PROC GetGUIDRows");
+	odbc_command_with_result(odbc_stmt, "DROP PROC GetGUIDRows");
 
 	/* CLOSEDOWN */
 
-	Disconnect();
+	odbc_disconnect();
 
 	AB_FUNCT(("TestRawODBCGuid (out): ok"));
 	return TRUE;
-
-      odbcfail:
-	DispODBCErrs(Environment, Connection, Statement);
-	DispODBCDiags(Statement);
-	AB_FUNCT(("TestRawODBCGuid (out): error"));
-	return FALSE;
 }
 
 /**
@@ -639,11 +426,9 @@ RunTests(void)
 int
 main(int argc, char *argv[])
 {
-	use_odbc_version3 = 1;
+	odbc_use_version3 = 1;
 
-	if (RunTests()) {
+	if (RunTests())
 		return 0;	/* Success */
-	} else {
-		return 1;	/* Error code */
-	}
+	return 1;	/* Error code */
 }
