@@ -1,5 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004  Brian Bruns
+ * Copyright (C) 2010  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,7 +31,7 @@
 #include "tds.h"
 #include "tdssrv.h"
 
-static char software_version[] = "$Id: server.c,v 1.24 2008/01/07 14:07:21 freddy77 Exp $";
+static char software_version[] = "$Id: server.c,v 1.27 2010/11/26 08:41:26 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 void
@@ -117,13 +118,17 @@ tds_send_msg(TDSSOCKET * tds, int msgno, int msgstate, int severity,
 	     const char *msgtext, const char *srvname, const char *procname, int line)
 {
 	int msgsz;
+	size_t len;
 
 	tds_put_byte(tds, TDS_INFO_TOKEN);
+	if (!procname)
+		procname = "";
+	len = strlen(procname);
 	msgsz = 4		/* msg no    */
 		+ 1		/* msg state */
 		+ 1		/* severity  */
 		/* FIXME ucs2 */
-		+ (IS_TDS7_PLUS(tds) ? 2 : 1) * (strlen(msgtext) + 1 + strlen(srvname) + 1 + strlen(procname))
+		+ (IS_TDS7_PLUS(tds) ? 2 : 1) * (strlen(msgtext) + 1 + strlen(srvname) + 1 + len)
 		+ 1 + 2;	/* line number */
 	tds_put_smallint(tds, msgsz);
 	tds_put_int(tds, msgno);
@@ -135,10 +140,10 @@ tds_send_msg(TDSSOCKET * tds, int msgno, int msgstate, int severity,
 	tds_put_byte(tds, strlen(srvname));
 	/* FIXME ucs2 */
 	tds_put_string(tds, srvname, strlen(srvname));
-	if (procname && strlen(procname)) {
-		tds_put_byte(tds, strlen(procname));
+	if (len) {
+		tds_put_byte(tds, len);
 		/* FIXME ucs2 */
-		tds_put_string(tds, procname, strlen(procname));
+		tds_put_string(tds, procname, len);
 	} else {
 		tds_put_byte(tds, 0);
 	}
@@ -162,8 +167,8 @@ tds_send_login_ack(TDSSOCKET * tds, const char *progname)
 		tds_put_byte(tds, 0);
 	} else {
 		tds_put_byte(tds, 1);
-		tds_put_byte(tds, tds->major_version);
-		tds_put_byte(tds, tds->minor_version);
+		tds_put_byte(tds, TDS_MAJOR(tds));
+		tds_put_byte(tds, TDS_MINOR(tds));
 	}
 	tds_put_byte(tds, 0);	/* unknown */
 	tds_put_byte(tds, 0);	/* unknown */
@@ -226,7 +231,7 @@ tds_send_done(TDSSOCKET * tds, int token, TDS_SMALLINT flags, TDS_INT numrows)
 	tds_put_byte(tds, token);
 	tds_put_smallint(tds, flags);
 	tds_put_smallint(tds, 2); /* are these two bytes the transaction status? */
-	if (IS_TDS90(tds))
+	if (IS_TDS72_PLUS(tds))
 		tds_put_int8(tds, numrows);
 	else
 		tds_put_int(tds, numrows);
@@ -395,7 +400,7 @@ tds7_send_result(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
  */
 void tds_send_table_header(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
 {
-	switch (tds->major_version) {
+	switch (TDS_MAJOR(tds)) {
 	case 4:
 		/*
 		 * TDS4 uses TDS_COLNAME_TOKEN to send column names, and
@@ -412,8 +417,6 @@ void tds_send_table_header(TDSSOCKET * tds, TDSRESULTINFO * resinfo)
 		break;
 
 	case 7:
-	case 8:
-	case 9:
 		/*
 		 * TDS7+ uses a TDS7_RESULT_TOKEN to send all column
 		 * information.
