@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
- * Copyright (C) 2005, 2006, 2007  Ziglio Frediano
+ * Copyright (C) 2005-2011  Ziglio Frediano
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,7 +39,7 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <process.h>
 #endif
 
@@ -51,7 +51,7 @@
 #include <dmalloc.h>
 #endif
 
-TDS_RCSID(var, "$Id: login.c,v 1.172 2007/12/31 10:06:50 freddy77 Exp $");
+TDS_RCSID(var, "$Id: login.c,v 1.202.2.1 2011/04/11 13:33:41 freddy77 Exp $");
 
 static int tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection);
 static int tds8_do_login(TDSSOCKET * tds, TDSCONNECTION * connection);
@@ -60,8 +60,7 @@ static int tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection);
 void
 tds_set_version(TDSLOGIN * tds_login, TDS_TINYINT major_ver, TDS_TINYINT minor_ver)
 {
-	tds_login->major_version = major_ver;
-	tds_login->minor_version = minor_ver;
+	tds_login->tds_version = ((TDS_USMALLINT) major_ver << 8) + minor_ver;
 }
 
 void
@@ -122,27 +121,27 @@ tds_set_app(TDSLOGIN * tds_login, const char *application)
 void
 tds_set_server(TDSLOGIN * tds_login, const char *server)
 {
-	if (!server || strlen(server) == 0) {
-		server = getenv("TDSQUERY");
-		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' from $TDSQUERY.\n", server);
-	}
-	if (!server || strlen(server) == 0) {
-		server = getenv("DSQUERY");
-		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' from $DSQUERY.\n", server);
-	}
-	if (!server || strlen(server) == 0) {
-		server = "SYBASE";
-		tdsdump_log(TDS_DBG_INFO1, "Setting 'server_name' to '%s' (compiled-in default).\n", server);
-	}
-	tds_dstr_copy(&tds_login->server_name, server);
-}
-
-void
-tds_set_server_addr(TDSLOGIN * tds_login, const char *server_addr)
-{
-	if (server_addr) {
-		tds_dstr_copy(&tds_login->server_addr, server_addr);
+#if 0
+	// Doing this in tds_alloc_login instead
+	static const char *names[] = { "TDSQUERY", "DSQUERY", "SYBASE" };
+	int i;
+	
+	for (i=0; i < TDS_VECTOR_SIZE(names) && (!server || strlen(server) == 0); i++) {
+		const char *source;
+		if (i + 1 == TDS_VECTOR_SIZE(names)) {
+			server = names[i];
+			source = "compiled-in default";
+		} else {
+			server = getenv(names[i]);
+			source = names[i];
+		}
+		if (server) {
+			tdsdump_log(TDS_DBG_INFO1, "Setting TDSLOGIN::server_name to '%s' from %s.\n", server, source);
+		}
 	}
+#endif
+	if (server)
+		tds_dstr_copy(&tds_login->server_name, server);
 }
 
 void
@@ -163,6 +162,12 @@ tds_set_language(TDSLOGIN * tds_login, const char *language)
 	tds_dstr_copy(&tds_login->language, language);
 }
 
+void
+tds_set_database_name(TDSLOGIN * tds_login, const char *dbname)
+{
+	tds_dstr_copy(&tds_login->database, dbname);
+}
+
 void
 tds_set_capabilities(TDSLOGIN * tds_login, unsigned char *capabilities, int size)
 {
@@ -306,64 +311,83 @@ free_save_context(TDSSAVECONTEXT *ctx)
  * Do a connection to socket
  * @param tds connection structure. This should be a non-connected connection.
  * @param connection info for connection
- * @return TDS_FAIL or TDS_SUCCEED
+ * @return TDS_FAIL or TDS_SUCCEED if a connection was made to the server's port.
+ * @return TDSERROR enumerated type if no TCP/IP connection could be formed. 
+ * @remark Possible error conditions:
+ *		- TDSESOCK: socket(2) failed: insufficient local resources
+ * 		- TDSECONN: connect(2) failed: invalid hostname or port (ETIMEDOUT, ECONNREFUSED, ENETUNREACH)
+ * 		- TDSEFCON: connect(2) succeeded, login packet not acknowledged.  
+ *		- TDS_FAIL: connect(2) succeeded, login failed.  
  */
-int
-tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
+static int
+tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection, int *p_oserr)
 {
-	int retval;
+	int erc = TDSEFCON;
 	int connect_timeout = 0;
 	int db_selected = 0;
-	char version[64];
-	char *str;
-	int len;
-	
+
 	/*
 	 * A major version of 0 means try to guess the TDS version. 
 	 * We try them in an order that should work. 
 	 */
-	const static struct tdsver {
-		TDS_TINYINT major_version;
-		TDS_TINYINT minor_version;
-	} versions[] =
-		{ { 8, 0 }
-		, { 7, 0 }
-		, { 5, 0 }
-		, { 4, 2 }
+	const static TDS_USMALLINT versions[] =
+		{ 0x702
+		, 0x701
+		, 0x700
+		, 0x500
+		, 0x402
 		};
 
-	if (connection->major_version == 0) {
+	/* disable tds9 if iconv wanted, currently not supported */
+	if (IS_TDS72_PLUS(connection) && tds->use_iconv) {
+		connection->tds_version = 0x701;
+		tdserror(tds->tds_ctx, tds, TDSEVERDOWN, 0);
+	}
+
+	if (TDS_MAJOR(connection) == 0) {
 		unsigned int i;
 		TDSSAVECONTEXT save_ctx;
 		const TDSCONTEXT *old_ctx = tds->tds_ctx;
 		typedef void (*env_chg_func_t) (TDSSOCKET * tds, int type, char *oldval, char *newval);
 		env_chg_func_t old_env_chg = tds->env_chg_func;
+		/* the context of a socket is const; we have to modify it to suppress error messages during multiple tries. */
+		TDSCONTEXT *mod_ctx = (TDSCONTEXT *)tds->tds_ctx;
+		err_handler_t err_handler = tds->tds_ctx->err_handler;
 
 		init_save_context(&save_ctx, old_ctx);
 		tds->tds_ctx = &save_ctx.ctx;
 		tds->env_chg_func = tds_save_env;
-		for (i=0; i < TDS_VECTOR_SIZE(versions); ++i) {
-			connection->major_version = versions[i].major_version;
-			connection->minor_version = versions[i].minor_version;
-			/* fprintf(stdout, "trying TDSVER %d.%d\n", connection->major_version, connection->minor_version); */
+		mod_ctx->err_handler = NULL;
+
+		for (i = tds->use_iconv? 1 : 0; i < TDS_VECTOR_SIZE(versions); ++i) {
+			connection->tds_version = versions[i];
 			reset_save_context(&save_ctx);
-			retval = tds_connect(tds, connection);
-			if (TDS_SUCCEED == retval)
+
+			if ((erc = tds_connect(tds, connection, p_oserr)) != TDS_SUCCEED) {
+				tds_close_socket(tds);
+			}
+			
+			if (erc != TDSEFCON)	/* TDSEFCON indicates wrong TDS version */
 				break;
-			tds_close_socket(tds);
 		}
+		
+		mod_ctx->err_handler = err_handler;
 		tds->env_chg_func = old_env_chg;
 		tds->tds_ctx = old_ctx;
 		replay_save_context(tds, &save_ctx);
 		free_save_context(&save_ctx);
-		return retval;
+		
+		if (erc != TDS_SUCCEED)
+			tdserror(tds->tds_ctx, tds, erc, *p_oserr);
+
+		return erc;
 	}
 	
 
 	/*
 	 * If a dump file has been specified, start logging
 	 */
-	if (!tds_dstr_isempty(&connection->dump_file)) {
+	if (!tds_dstr_isempty(&connection->dump_file) && !tdsdump_isopen()) {
 		if (connection->debug_flags)
 			tds_debug_flags = connection->debug_flags;
 		tdsdump_open(tds_dstr_cstr(&connection->dump_file));
@@ -371,8 +395,7 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 
 	tds->connection = connection;
 
-	tds->major_version = connection->major_version;
-	tds->minor_version = connection->minor_version;
+	tds->tds_version = connection->tds_version;
 	tds->emul_little_endian = connection->emul_little_endian;
 #ifdef WORDS_BIGENDIAN
 	if (IS_TDS7_PLUS(tds)) {
@@ -388,12 +411,6 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 		}
 	}
 
-	/* specified a date format? */
-	/*
-	 * if (connection->date_fmt) {
-	 * tds->date_fmt=strdup(connection->date_fmt);
-	 * }
-	 */
 	connect_timeout = connection->connect_timeout;
 
 	/* Jeff's hack - begin */
@@ -402,51 +419,60 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 
 	/* verify that ip_addr is not empty */
 	if (tds_dstr_isempty(&connection->ip_addr)) {
+		tdserror(tds->tds_ctx, tds, TDSEUHST, 0 );
 		tdsdump_log(TDS_DBG_ERROR, "IP address pointer is empty\n");
 		if (!tds_dstr_isempty(&connection->server_name)) {
 			tdsdump_log(TDS_DBG_ERROR, "Server %s not found!\n", tds_dstr_cstr(&connection->server_name));
 		} else {
 			tdsdump_log(TDS_DBG_ERROR, "No server specified!\n");
 		}
-		return TDS_FAIL;
+		return TDSECONN;
 	}
 
-	if (!IS_TDS50(tds) && !tds_dstr_isempty(&connection->instance_name))
+	if (!IS_TDS50(tds) && !tds_dstr_isempty(&connection->instance_name) && !connection->port)
 		connection->port = tds7_get_instance_port(tds_dstr_cstr(&connection->ip_addr), tds_dstr_cstr(&connection->instance_name));
 
 	if (connection->port < 1) {
 		tdsdump_log(TDS_DBG_ERROR, "invalid port number\n");
-		return TDS_FAIL;
+		return TDSECONN;
 	}
 
 	memcpy(tds->capabilities, connection->capabilities, TDS_MAX_CAPABILITY);
 
-	retval = tds_version(tds, version);
-	if (!retval)
-		version[0] = '\0';
-
-	if (tds_open_socket(tds, tds_dstr_cstr(&connection->ip_addr), connection->port, connect_timeout) != TDS_SUCCEED)
-		return TDS_FAIL;
+	if ((erc = tds_open_socket(tds, tds_dstr_cstr(&connection->ip_addr), connection->port, connect_timeout, p_oserr)) != TDSEOK) {
+		tdserror(tds->tds_ctx, tds, erc, *p_oserr);
+		return erc;
+	}
+		
+	/*
+	 * Beyond this point, we're connected to the server.  We know we have a valid TCP/IP address+socket pair.  
+	 * Although network errors *might* happen, most problems from here on out will be TDS-level errors, 
+	 * either TDS version problems or authentication problems.  
+	 */
+		
 	tds_set_state(tds, TDS_IDLE);
 
-	if (IS_TDS8_PLUS(tds)) {
-		retval = tds8_do_login(tds, connection);
+	if (IS_TDS71_PLUS(tds)) {
+		erc = tds8_do_login(tds, connection);
 		db_selected = 1;
 	} else if (IS_TDS7_PLUS(tds)) {
-		retval = tds7_send_login(tds, connection);
+		erc = tds7_send_login(tds, connection);
 		db_selected = 1;
 	} else {
 		tds->out_flag = TDS_LOGIN;
-		retval = tds_send_login(tds, connection);
+		erc = tds_send_login(tds, connection);
 	}
-	if (retval == TDS_FAIL || !tds_process_login_tokens(tds)) {
+	if (erc == TDS_FAIL || !tds_process_login_tokens(tds)) {
+		tdsdump_log(TDS_DBG_ERROR, "login packet %s\n", erc==TDS_SUCCEED? "accepted":"rejected");
 		tds_close_socket(tds);
 		tdserror(tds->tds_ctx, tds, TDSEFCON, 0); 	/* "Adaptive Server connection failed" */
-								/* If it's a bad login, the server will send that mesasge */
-		return TDS_FAIL;
+		return TDSEFCON;
 	}
 
 	if (connection->text_size || (!db_selected && !tds_dstr_isempty(&connection->database))) {
+		char *str;
+		int len;
+
 		len = 64 + tds_quote_id(tds, NULL, tds_dstr_cstr(&connection->database),-1);
 		if ((str = (char *) malloc(len)) == NULL)
 			return TDS_FAIL;
@@ -459,9 +485,9 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 			strcat(str, "use ");
 			tds_quote_id(tds, strchr(str, 0), tds_dstr_cstr(&connection->database), -1);
 		}
-		retval = tds_submit_query(tds, str);
+		erc = tds_submit_query(tds, str);
 		free(str);
-		if (retval != TDS_SUCCEED)
+		if (erc != TDS_SUCCEED)
 			return TDS_FAIL;
 
 		if (tds_process_simple_query(tds) != TDS_SUCCEED)
@@ -473,11 +499,17 @@ tds_connect(TDSSOCKET * tds, TDSCONNECTION * connection)
 	return TDS_SUCCEED;
 }
 
+int
+tds_connect_and_login(TDSSOCKET * tds, TDSCONNECTION * connection)
+{
+	int oserr = 0;
+	return tds_connect(tds, connection, &oserr);
+}
+
 static int
 tds_put_login_string(TDSSOCKET * tds, const char *buf, int n)
 {
-	int buf_len = (buf ? strlen(buf) : 0);
-
+	const int buf_len = buf ? (int)strlen(buf) : 0;
 	return tds_put_buf(tds, (const unsigned char *) buf, n, buf_len);
 }
 
@@ -533,6 +565,9 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	int len;
 	char blockstr[16];
 
+	/* override lservname field for ASA servers */	
+	const char *lservname = getenv("ASA_DATABASE")? getenv("ASA_DATABASE") : tds_dstr_cstr(&connection->server_name);
+
 	if (strchr(tds_dstr_cstr(&connection->user_name), '\\') != NULL) {
 		tdsdump_log(TDS_DBG_ERROR, "NT login not support using TDS 4.x or 5.0\n");
 		return TDS_FAIL;
@@ -549,7 +584,7 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 		memcpy(program_version, "\005\000\000\000", 4);
 	} else {
 		tdsdump_log(TDS_DBG_SEVERE, "Unknown protocol version!\n");
-		exit(1);
+		return TDS_FAIL;
 	}
 	/*
 	 * the following code is adapted from  Arno Pedusaar's 
@@ -560,7 +595,8 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	tds_put_login_string(tds, tds_dstr_cstr(&connection->client_host_name), TDS_MAX_LOGIN_STR_SZ);	/* client host name */
 	tds_put_login_string(tds, tds_dstr_cstr(&connection->user_name), TDS_MAX_LOGIN_STR_SZ);	/* account name */
 	tds_put_login_string(tds, tds_dstr_cstr(&connection->password), TDS_MAX_LOGIN_STR_SZ);	/* account password */
-	tds_put_login_string(tds, "37876", TDS_MAX_LOGIN_STR_SZ);	/* host process */
+	sprintf(blockstr, "%d", (int) getpid());
+	tds_put_login_string(tds, blockstr, TDS_MAX_LOGIN_STR_SZ);	/* host process */
 #ifdef WORDS_BIGENDIAN
 	if (tds->emul_little_endian) {
 		tds_put_n(tds, le1, 6);
@@ -579,11 +615,11 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	}
 	tds_put_n(tds, magic3, 3);
 	tds_put_login_string(tds, tds_dstr_cstr(&connection->app_name), TDS_MAX_LOGIN_STR_SZ);
-	tds_put_login_string(tds, tds_dstr_cstr(&connection->server_name), TDS_MAX_LOGIN_STR_SZ);
+	tds_put_login_string(tds, lservname, TDS_MAX_LOGIN_STR_SZ);
 	if (IS_TDS42(tds)) {
 		tds_put_login_string(tds, tds_dstr_cstr(&connection->password), 255);
 	} else {
-		len = tds_dstr_len(&connection->password);
+		len = (int)tds_dstr_len(&connection->password);
 		if (len > 253)
 			len = 0;
 		tds_put_byte(tds, 0);
@@ -629,7 +665,7 @@ tds_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	tds_put_byte(tds, 1);
 
 	/* network packet size */
-	if (connection->block_size < 1000000 && connection->block_size)
+	if (connection->block_size < 65536 && connection->block_size > 0)
 		sprintf(blockstr, "%d", connection->block_size);
 	else
 		strcpy(blockstr, "512");
@@ -661,13 +697,13 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 
 	static const unsigned char client_progver[] = { 6, 0x83, 0xf2, 0xf8 };
 
-	static const unsigned char tds7Version[] = { 0x00, 0x00, 0x00, 0x70 };
-	static const unsigned char tds8Version[] = { 0x01, 0x00, 0x00, 0x71 };
-	static const unsigned char tds9Version[] = { 0x02, 0x00, 0x09, 0x72 };
+	static const unsigned char tds70Version[] = { 0x00, 0x00, 0x00, 0x70 };
+	static const unsigned char tds71Version[] = { 0x01, 0x00, 0x00, 0x71 };
+	static const unsigned char tds72Version[] = { 0x02, 0x00, 0x09, 0x72 };
 
 	static const unsigned char connection_id[] = { 0x00, 0x00, 0x00, 0x00 };
 	unsigned char option_flag1 = 0x00;
-	unsigned char option_flag2 = tds->option_flag2;
+	unsigned char option_flag2 = connection->option_flag2;
 	static const unsigned char sql_type_flag = 0x00;
 	static const unsigned char reserved_flag = 0x00;
 
@@ -680,20 +716,21 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	char unicode_string[256];
 	char *punicode;
 	size_t unicode_left;
-	int packet_size;
-	int block_size;
-	int current_pos;
+	size_t packet_size;
+	TDS_INT block_size;
+	size_t current_pos;
 
 	const char *user_name = tds_dstr_cstr(&connection->user_name);
-	int user_name_len = strlen(user_name);
-	int host_name_len = tds_dstr_len(&connection->client_host_name);
-	int app_name_len = tds_dstr_len(&connection->app_name);
+	// FIXME: These are defined as size_t, but should be TDS_SMALLINT. 
+	size_t user_name_len = strlen(user_name);
+	size_t host_name_len = tds_dstr_len(&connection->client_host_name);
+	size_t app_name_len = tds_dstr_len(&connection->app_name);
 	size_t password_len = tds_dstr_len(&connection->password);
-	int server_name_len = tds_dstr_len(&connection->server_name);
-	int library_len = tds_dstr_len(&connection->library);
-	int language_len = tds_dstr_len(&connection->language);
-	int database_len = tds_dstr_len(&connection->database);
-	int auth_len = 0;
+	size_t server_name_len = tds_dstr_len(&connection->server_name);
+	size_t library_len = tds_dstr_len(&connection->library);
+	size_t language_len = tds_dstr_len(&connection->language);
+	size_t database_len = tds_dstr_len(&connection->database);
+	size_t auth_len = 0;
 
 	tds->out_flag = TDS7_LOGIN;
 
@@ -707,11 +744,19 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	if (password_len > 128)
 		password_len = 128;
 
-	current_pos = IS_TDS90(tds) ? 86 + 8 : 86;	/* ? */
+	current_pos = IS_TDS72_PLUS(tds) ? 86 + 8 : 86;	/* ? */
 
 	packet_size = current_pos + (host_name_len + app_name_len + server_name_len + library_len + language_len + database_len) * 2;
 
 	/* check ntlm */
+#ifdef HAVE_SSPI
+	if (strchr(user_name, '\\') != NULL || user_name_len == 0) {
+		tds->authentication = tds_sspi_get_auth(tds);
+		if (!tds->authentication)
+			return TDS_FAIL;
+		auth_len = tds->authentication->packet_len;
+		packet_size += auth_len;
+#else
 	if (strchr(user_name, '\\') != NULL) {
 		tds->authentication = tds_ntlm_get_auth(tds);
 		if (!tds->authentication)
@@ -719,29 +764,31 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 		auth_len = tds->authentication->packet_len;
 		packet_size += auth_len;
 	} else if (user_name_len == 0) {
-#ifdef ENABLE_KRB5
+# ifdef ENABLE_KRB5
 		/* try kerberos */
 		tds->authentication = tds_gss_get_auth(tds);
 		if (!tds->authentication)
 			return TDS_FAIL;
 		auth_len = tds->authentication->packet_len;
 		packet_size += auth_len;
-#else
+# else
 		return TDS_FAIL;
+# endif
 #endif
 	} else
 		packet_size += (user_name_len + password_len) * 2;
 
+#if !defined(TDS_DEBUG_LOGIN)
 	tdsdump_log(TDS_DBG_INFO2, "quietly sending TDS 7+ login packet\n");
 	tdsdump_off();
-
-	tds_put_int(tds, packet_size);
-	if (IS_TDS90(tds)) {
-		tds_put_n(tds, tds9Version, 4);
-	} else if (IS_TDS8_PLUS(tds)) {
-		tds_put_n(tds, tds8Version, 4);
+#endif
+	TDS_PUT_INT(tds, packet_size);
+	if (IS_TDS72_PLUS(tds)) {
+		tds_put_n(tds, tds72Version, 4);
+	} else if (IS_TDS71_PLUS(tds)) {
+		tds_put_n(tds, tds71Version, 4);
 	} else {
-		tds_put_n(tds, tds7Version, 4);
+		tds_put_n(tds, tds70Version, 4);
 	}
 
 	if (connection->block_size < 1000000 && connection->block_size >= 512)
@@ -777,8 +824,8 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	tds_put_n(tds, collation, 4);
 
 	/* host name */
-	tds_put_smallint(tds, current_pos);
-	tds_put_smallint(tds, host_name_len);
+	TDS_PUT_SMALLINT(tds, current_pos);
+	TDS_PUT_SMALLINT(tds, host_name_len);
 	current_pos += host_name_len * 2;
 	if (tds->authentication) {
 		tds_put_smallint(tds, 0);
@@ -787,36 +834,36 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 		tds_put_smallint(tds, 0);
 	} else {
 		/* username */
-		tds_put_smallint(tds, current_pos);
-		tds_put_smallint(tds, user_name_len);
+		TDS_PUT_SMALLINT(tds, current_pos);
+		TDS_PUT_SMALLINT(tds, user_name_len);
 		current_pos += user_name_len * 2;
 		/* password */
-		tds_put_smallint(tds, current_pos);
-		tds_put_smallint(tds, password_len);
+		TDS_PUT_SMALLINT(tds, current_pos);
+		TDS_PUT_SMALLINT(tds, password_len);
 		current_pos += password_len * 2;
 	}
 	/* app name */
-	tds_put_smallint(tds, current_pos);
-	tds_put_smallint(tds, app_name_len);
+	TDS_PUT_SMALLINT(tds, current_pos);
+	TDS_PUT_SMALLINT(tds, app_name_len);
 	current_pos += app_name_len * 2;
 	/* server name */
-	tds_put_smallint(tds, current_pos);
-	tds_put_smallint(tds, server_name_len);
+	TDS_PUT_SMALLINT(tds, current_pos);
+	TDS_PUT_SMALLINT(tds, server_name_len);
 	current_pos += server_name_len * 2;
 	/* unknown */
 	tds_put_smallint(tds, 0);
 	tds_put_smallint(tds, 0);
 	/* library name */
-	tds_put_smallint(tds, current_pos);
-	tds_put_smallint(tds, library_len);
+	TDS_PUT_SMALLINT(tds, current_pos);
+	TDS_PUT_SMALLINT(tds, library_len);
 	current_pos += library_len * 2;
 	/* language  - kostya@warmcat.excom.spb.su */
-	tds_put_smallint(tds, current_pos);
-	tds_put_smallint(tds, language_len);
+	TDS_PUT_SMALLINT(tds, current_pos);
+	TDS_PUT_SMALLINT(tds, language_len);
 	current_pos += language_len * 2;
 	/* database name */
-	tds_put_smallint(tds, current_pos);
-	tds_put_smallint(tds, database_len);
+	TDS_PUT_SMALLINT(tds, current_pos);
+	TDS_PUT_SMALLINT(tds, database_len);
 	current_pos += database_len * 2;
 
 	/* MAC address */
@@ -824,27 +871,29 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	tds_put_n(tds, hwaddr, 6);
 
 	/* authentication stuff */
-	tds_put_smallint(tds, current_pos);
-	tds_put_smallint(tds, auth_len);	/* this matches numbers at end of packet */
+	TDS_PUT_SMALLINT(tds, current_pos);
+	TDS_PUT_SMALLINT(tds, auth_len);	/* this matches numbers at end of packet */
 	current_pos += auth_len;
 
-	/* unknown */
-	tds_put_smallint(tds, current_pos);
+	/* db file */
+	TDS_PUT_SMALLINT(tds, current_pos);
 	tds_put_smallint(tds, 0);
 
-	if (IS_TDS90(tds)) {
-		tds_put_smallint(tds, current_pos);
+	if (IS_TDS72_PLUS(tds)) {
+		/* new password */
+		TDS_PUT_SMALLINT(tds, current_pos);
 		tds_put_smallint(tds, 0);
 
+		/* SSPI long */
 		tds_put_int(tds, 0);
 	}
 
 	/* FIXME here we assume single byte, do not use *2 to compute bytes, convert before !!! */
-	tds_put_string(tds, tds_dstr_cstr(&connection->client_host_name), host_name_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->client_host_name), (int)host_name_len);
 	if (!tds->authentication) {
 		const char *p;
 		TDSICONV *char_conv = tds->char_convs[client2ucs2];
-		tds_put_string(tds, tds_dstr_cstr(&connection->user_name), user_name_len);
+		tds_put_string(tds, tds_dstr_cstr(&connection->user_name), (int)user_name_len);
 		p = tds_dstr_cstr(&connection->password);
 		punicode = unicode_string;
 		unicode_left = sizeof(unicode_string);
@@ -859,11 +908,11 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 		tds7_crypt_pass((unsigned char *) unicode_string, password_len, (unsigned char *) unicode_string);
 		tds_put_n(tds, unicode_string, password_len);
 	}
-	tds_put_string(tds, tds_dstr_cstr(&connection->app_name), app_name_len);
-	tds_put_string(tds, tds_dstr_cstr(&connection->server_name), server_name_len);
-	tds_put_string(tds, tds_dstr_cstr(&connection->library), library_len);
-	tds_put_string(tds, tds_dstr_cstr(&connection->language), language_len);
-	tds_put_string(tds, tds_dstr_cstr(&connection->database), database_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->app_name), (int)app_name_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->server_name), (int)server_name_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->library), (int)library_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->language), (int)language_len);
+	tds_put_string(tds, tds_dstr_cstr(&connection->database), (int)database_len);
 
 	if (tds->authentication)
 		tds_put_n(tds, tds->authentication->packet, auth_len);
@@ -880,9 +929,9 @@ tds7_send_login(TDSSOCKET * tds, TDSCONNECTION * connection)
  * 'len' characters
  */
 unsigned char *
-tds7_crypt_pass(const unsigned char *clear_pass, int len, unsigned char *crypt_pass)
+tds7_crypt_pass(const unsigned char *clear_pass, size_t len, unsigned char *crypt_pass)
 {
-	int i;
+	size_t i;
 
 	for (i = 0; i < len; i++)
 		crypt_pass[i] = ((clear_pass[i] << 4) | (clear_pass[i] >> 4)) ^ 0xA5;
@@ -892,18 +941,14 @@ tds7_crypt_pass(const unsigned char *clear_pass, int len, unsigned char *crypt_p
 static int
 tds8_do_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 {
-#if !defined(HAVE_GNUTLS) && !defined(HAVE_OPENSSL)
-	/*
-	 * In case we do not have SSL enabled do not send pre-login so
-	 * if server has certificate but no force encryption login success
-	 */
-	return tds7_send_login(tds, connection);
-#else
-	int i, len, ret;
+	int i, len;
 	const char *instance_name = tds_dstr_isempty(&connection->instance_name) ? "MSSQLServer" : tds_dstr_cstr(&connection->instance_name);
 	int instance_name_len = strlen(instance_name) + 1;
 	TDS_CHAR crypt_flag;
 	unsigned int start_pos = 21;
+#if defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	int ret;
+#endif
 
 #define START_POS 21
 #define UI16BE(n) ((n) >> 8), ((n) & 0xffu)
@@ -928,7 +973,7 @@ tds8_do_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	TDS_UCHAR *p;
 
 	SET_UI16BE(13, instance_name_len);
-	if (!IS_TDS90(tds)) {
+	if (!IS_TDS72_PLUS(tds)) {
 		SET_UI16BE(16, START_POS + 6 + 1 + instance_name_len);
 		buf[20] = 0xff;
 	} else {
@@ -945,20 +990,32 @@ tds8_do_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 	assert(start_pos >= 21 && start_pos <= sizeof(buf));
 	assert(buf[start_pos-1] == 0xff);
 
+	/*
+	 * fix a problem with mssql2k which doesn't like
+	 * packet splitted during SSL handshake
+	 */
+	if (tds->env.block_size < 4096)
+		tds_realloc_socket(tds, 4096);
+
 	/* do prelogin */
 	tds->out_flag = TDS8_PRELOGIN;
 
 	tds_put_n(tds, buf, start_pos);
 	/* netlib version */
-	tds_put_n(tds, IS_TDS90(tds) ? netlib9 : netlib8, 6);
+	tds_put_n(tds, IS_TDS72_PLUS(tds) ? netlib9 : netlib8, 6);
 	/* encryption */
-	tds_put_byte(tds, connection->encryption_level ? 1 : 0);
+#if !defined(HAVE_GNUTLS) && !defined(HAVE_OPENSSL)
+	/* not supported */
+	tds_put_byte(tds, 2);
+#else
+	tds_put_byte(tds, connection->encryption_level >= TDS_ENCRYPTION_REQUIRE ? 1 : 0);
+#endif
 	/* instance */
 	tds_put_n(tds, instance_name, instance_name_len);
 	/* pid */
 	tds_put_int(tds, getpid());
-	/* ???? unknown ??? */
-	if (IS_TDS90(tds))
+	/* MARS (1 enabled) */
+	if (IS_TDS72_PLUS(tds))
 		tds_put_byte(tds, 0);
 	if (tds_flush_packet(tds) == TDS_FAIL)
 		return TDS_FAIL;
@@ -1007,7 +1064,10 @@ tds8_do_login(TDSSOCKET * tds, TDSCONNECTION * connection)
 
 		return tds7_send_login(tds, connection);
 	}
-
+#if !defined(HAVE_GNUTLS) && !defined(HAVE_OPENSSL)
+	tdsdump_log(TDS_DBG_ERROR, "server required encryption but support is not compiled in\n");
+	return TDS_FAIL;
+#else
 	/*
 	 * if server has a certificate it require at least a crypted login
 	 * (even if data is not encrypted)
