@@ -42,7 +42,7 @@
 #include "tds.h"
 #include "md4.h"
 
-TDS_RCSID(var, "$Id: md4.c,v 1.7 2005/07/07 13:06:45 freddy77 Exp $");
+TDS_RCSID(var, "$Id: md4.c,v 1.8 2009/01/16 20:27:58 jklowden Exp $");
 
 #undef word32
 #define word32 TDS_UINT
@@ -92,7 +92,7 @@ MD4Init(struct MD4Context *ctx)
  * of bytes.
  */
 void
-MD4Update(struct MD4Context *ctx, unsigned char const *buf, unsigned len)
+MD4Update(struct MD4Context *ctx, unsigned char const *buf, size_t len)
 {
 	register word32 t;
 
@@ -101,7 +101,7 @@ MD4Update(struct MD4Context *ctx, unsigned char const *buf, unsigned len)
 	t = ctx->bits[0];
 	if ((ctx->bits[0] = t + ((word32) len << 3)) < t)
 		ctx->bits[1]++;	/* Carry from low to high */
-	ctx->bits[1] += len >> 29;
+	ctx->bits[1] += (TDS_UINT)len >> 29;
 
 	t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
 
