@@ -1,5 +1,5 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
- * Copyright (C) 2003  Craig A. Berry	craigberry@mac.com	23-JAN-2003
+ * Copyright (C) 2003, 2010  Craig A. Berry	craigberry@mac.com
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -40,12 +40,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
+#include <unistd.h>
 
 #include <replacements/readpassphrase.h>
+#include "readline.h"
 
-static char software_version[] = "$Id: getpass.c,v 1.5 2005/12/29 10:24:34 freddy77 Exp $";
+static FILE *tds_rl_instream = NULL;
+static FILE *tds_rl_outstream = NULL;
+
+static char software_version[] = "$Id: getpass.c,v 1.7 2010/12/17 04:26:21 berryc Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-static char passbuff[128];
 
 /* 
  * A collection of assorted UNIXy input functions for VMS.  The core
@@ -60,7 +64,7 @@ static char passbuff[128];
  * already stored in the command recall buffer by SMG$READ_COMPOSED_LINE.
  */
 
-#define MY_PASSWORD_LEN 1024
+#define MY_PASSWORD_LEN 8192
 #define RECALL_SIZE     50	/* Lines in recall buffer. */
 #define DEFAULT_TIMEOUT 30	/* Seconds to wait for user input. */
 
@@ -77,15 +81,16 @@ readpassphrase(const char *prompt, char *pbuf, size_t buflen, int flags)
 	unsigned long ctrl_mask, saved_ctrl_mask = 0;
 	int timeout_secs = 0;
 	int *timeout_ptr = NULL;
-	unsigned long dvi_item, ttdevclass, status = 0;
+	unsigned long status = 0;
 	unsigned short iosb[4];
-	unsigned short ttchan, result_len = 0;
+	unsigned short ttchan, result_len = 0, stdin_is_tty;
 
-	$DESCRIPTOR(ttdsc, "SYS$COMMAND:");
+	$DESCRIPTOR(ttdsc, "");
 	$DESCRIPTOR(pbuf_dsc, "");
 	$DESCRIPTOR(prompt_dsc, "");
 	char *retval = NULL;
 	char *myprompt = NULL;
+	char input_fspec[MY_PASSWORD_LEN + 1];
 
 	if (pbuf == NULL || buflen == 0) {
 		errno = EINVAL;
@@ -95,24 +100,25 @@ readpassphrase(const char *prompt, char *pbuf, size_t buflen, int flags)
 	pbuf_dsc.dsc$a_pointer = pbuf;
 	pbuf_dsc.dsc$w_length = buflen - 1;
 
+
 	/*
-	 * Find out if SYS$COMMAND is a terminal.
+	 * If stdin is not a terminal and only reading from a terminal is allowed, we
+	 * stop here.  
 	 */
-	dvi_item = DVI$_DEVCLASS;
-	status = LIB$GETDVI(&dvi_item, 0, &ttdsc, &ttdevclass);
-	if (!$VMS_STATUS_SUCCESS(status)) {
-		/* We might fail for perfectly good reasons, like
-		 * SYS$COMMAND is not a device. 
-		 */
-		ttdevclass = 0;
+	stdin_is_tty = isatty(fileno(stdin));
+	if (stdin_is_tty != 1 && (flags & RPP_REQUIRE_TTY)) {
+		errno = ENOTTY;
+		return NULL;
 	}
 
 	/*
-	 * If it's not a terminal and only reading from a terminal is allowed, we
-	 * stop here.  
+	 * We need the file or device associated with stdin in VMS format.
 	 */
-	if ((ttdevclass != DC$_TERM) && (flags & RPP_REQUIRE_TTY)) {
-		errno = ENOTTY;
+	if (fgetname(stdin, input_fspec, 1)) {
+		ttdsc.dsc$a_pointer = (char *)&input_fspec;
+		ttdsc.dsc$w_length = strlen(input_fspec);
+	} else {
+		errno = EMFILE;
 		return NULL;
 	}
 
@@ -128,14 +134,16 @@ readpassphrase(const char *prompt, char *pbuf, size_t buflen, int flags)
 	prompt_dsc.dsc$a_pointer = myprompt;
 	prompt_dsc.dsc$w_length = strlen(myprompt);
 
-	/* Disable Ctrl-T and Ctrl-Y */
-	ctrl_mask = LIB$M_CLI_CTRLT | LIB$M_CLI_CTRLY;
-	status = LIB$DISABLE_CTRL(&ctrl_mask, &saved_ctrl_mask);
-	if (!$VMS_STATUS_SUCCESS(status)) {
-		errno = EVMSERR;
-		vaxc$errno = status;
-		free(myprompt);
-		return NULL;
+	if (!(flags & RPP_ECHO_ON) && (stdin_is_tty)) {
+		/* Disable Ctrl-T and Ctrl-Y */
+		ctrl_mask = LIB$M_CLI_CTRLT | LIB$M_CLI_CTRLY;
+		status = LIB$DISABLE_CTRL(&ctrl_mask, &saved_ctrl_mask);
+		if (!$VMS_STATUS_SUCCESS(status)) {
+			errno = EVMSERR;
+			vaxc$errno = status;
+			free(myprompt);
+			return NULL;
+		}
 	}
 
 	/* 
@@ -151,7 +159,7 @@ readpassphrase(const char *prompt, char *pbuf, size_t buflen, int flags)
 		timeout_ptr = &timeout_secs;
 	}
 
-	if (!(flags & RPP_ECHO_ON) && (ttdevclass == DC$_TERM)) {
+	if (!(flags & RPP_ECHO_ON) && (stdin_is_tty)) {
 		/* 
 		 * If we are suppressing echoing, get a line of input with $QIOW.  
 		 * Non-echoed lines are not stored for recall.  (The same thing
@@ -161,7 +169,7 @@ readpassphrase(const char *prompt, char *pbuf, size_t buflen, int flags)
 		status = SYS$ASSIGN(&ttdsc, &ttchan, 0, 0, 0);
 		if ($VMS_STATUS_SUCCESS(status)) {
 
-			unsigned long qio_func = IO$_READPROMPT | IO$M_NOECHO;
+			unsigned long qio_func = IO$_READPROMPT | IO$M_NOECHO | IO$M_PURGE;
 
 			if (!(flags & RPP_TIMEOUT_OFF))
 				qio_func |= IO$M_TIMED;
@@ -239,15 +247,17 @@ readpassphrase(const char *prompt, char *pbuf, size_t buflen, int flags)
 
 	free(myprompt);
 
-	/* 
-	 * Reenable previous control processing.
-	 */
-	status = LIB$ENABLE_CTRL(&saved_ctrl_mask);
+	if (!(flags & RPP_ECHO_ON) && (stdin_is_tty)) {
+		/*
+		 * Reenable previous control processing.
+		 */
+		status = LIB$ENABLE_CTRL(&saved_ctrl_mask);
 
-	if (!$VMS_STATUS_SUCCESS(status)) {
-		errno = EVMSERR;
-		vaxc$errno = status;
-		return NULL;
+		if (!$VMS_STATUS_SUCCESS(status)) {
+			errno = EVMSERR;
+			vaxc$errno = status;
+			return NULL;
+		}
 	}
 
 	return retval;
@@ -271,10 +281,12 @@ getpass(const char *prompt)
 char *
 readline(char *prompt)
 {
-
-	char *buf = NULL;
-	char *s = readpassphrase((const char *) prompt, passbuf, sizeof(passbuf),
-				 RPP_ECHO_ON | RPP_TIMEOUT_OFF);
+	char *buf = NULL, *s = NULL, *p = NULL;
+	if (tds_rl_instream == NULL)
+		s = readpassphrase((const char *) prompt, passbuf, sizeof(passbuf),
+					 RPP_ECHO_ON | RPP_TIMEOUT_OFF);
+	else
+		s = fgets(passbuf, sizeof(passbuf), tds_rl_instream);
 
 	if (s != NULL) {
 		buf = (char *) malloc(strlen(s) + 1);
@@ -288,3 +300,17 @@ void
 add_history(const char *s)
 {
 }
+
+
+FILE **
+rl_instream_get_addr(void)
+{
+	return &tds_rl_instream;
+}
+
+FILE **
+rl_outstream_get_addr(void)
+{
+	return &tds_rl_outstream;
+}
+
