@@ -20,7 +20,7 @@
 #ifndef _tds_sysdep_public_h_
 #define _tds_sysdep_public_h_
 
-static char rcsid_tds_sysdep_public_h[] = "$Id: tds_sysdep_public.h,v 1.5.6.1 2008/01/23 08:50:17 freddy77 Exp $";
+static char rcsid_tds_sysdep_public_h[] = "$Id: tds_sysdep_public.h,v 1.8.2.1 2011/08/04 04:38:20 freddy77 Exp $";
 static void *no_unused_tds_sysdep_public_h_warn[] = { rcsid_tds_sysdep_public_h, no_unused_tds_sysdep_public_h_warn };
 
 #ifdef __cplusplus
@@ -28,6 +28,7 @@ extern "C"
 {
 #endif
 
+#include <winsock2.h>
 #include <windows.h>
 #define tds_sysdep_int16_type short	/* 16-bit int */
 #define tds_sysdep_int32_type int	/* 32-bit int */
@@ -39,10 +40,6 @@ extern "C"
 #else
 #define tds_sysdep_intptr_type __int64	/* 64-bit int */
 #endif
-typedef SOCKET TDS_SYS_SOCKET;
-#ifndef TDS_IS_SOCKET_INVALID
-#define TDS_IS_SOCKET_INVALID(s) ((s) == INVALID_SOCKET)
-#endif
 
 #if !defined(MSDBLIB) && !defined(SYBDBLIB)
 #define SYBDBLIB 1
